/*
    This file is part of libkcal.
    Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#ifndef KCAL_FREEBUSY_H
#define KCAL_FREEBUSY_H
//
// FreeBusy - information about free/busy times
//

#include <qdatetime.h>
#include <qvaluelist.h>
#include <qptrlist.h>

#include "period.h"
#include "calendar.h"

#include "incidencebase.h"

namespace KCal {

/**
  This class provides information about free/busy time of a calendar user.
*/
class FreeBusy : public IncidenceBase
{
  public:
    FreeBusy();
    FreeBusy(QDateTime start, QDateTime end);
    FreeBusy(Calendar *calendar, const QDateTime &start, const QDateTime &end);
    FreeBusy(QValueList<Period> busyPeriods);

    ~FreeBusy();
    
    QCString type() const { return "FreeBusy"; }

    QDateTime dtEnd();
    bool setDtEnd(QDateTime end);

    QValueList<Period> busyPeriods() const;

    void addPeriod(QDateTime start, QDateTime end);
    void sortList();
    
  private:

    //This is used for creating a freebusy object for the current user
    bool addLocalPeriod(QDateTime start, QDateTime end);

    QDateTime mDtEnd;
    QValueList<Period> mBusyPeriods;
    Calendar *mCalendar;
};

}

#endif
